@@ -1,7 +1,7 @@
 '''Frame is the Client Framework that verifies and installs entries, and generates statistics'''
-__revision__ = '$Revision: 4378 $'
+__revision__ = '$Revision: 4933 $'
 
-import logging, time
+import logging, time, types
 import Bcfg2.Client.Tools
 
 def cmpent(ent1, ent2):
@@ -27,18 +27,22 @@ def promptFilter(prompt, entries):
             continue
     return ret
 
-toolset_defs = {'rh': {'Service':'chkconfig', 'Package':'rpm'},
-                'debian': {'Service':'deb', 'Package':'deb'},
-                'nexenta': {'Service':'smf', 'Package':'deb'},
-                'solaris': {'Service':'smf', 'Package':'sysv'},
-                'gentoo': {'Service':'rc-update', 'Package':'ebuild'},
-                'freebsd': {'Package':'freebsdpkg'}}
+def matches_white_list(entry, whitelist):
+    return [entry.tag, entry.get('name')] in whitelist or \
+           [entry.tag, '*'] in whitelist or \
+           ['*', entry.get('name')] in whitelist
+
+def passes_black_list(entry, blacklist):
+    return [entry.tag, entry.get('name')] not in blacklist and \
+           [entry.tag, '*'] not in blacklist and \
+           ['*', entry.get('name')] not in blacklist
 
 class Frame:
     '''Frame is the container for all Tool objects and state information'''
-    def __init__(self, config, setup, times):
+    def __init__(self, config, setup, times, drivers, dryrun):
         self.config = config
         self.times = times
+        self.dryrun = dryrun
         self.times['initialization'] = time.time()
         self.setup = setup
         self.tools = []
@@ -47,18 +51,16 @@ class Frame:
         self.blacklist = []
         self.removal = []
         self.logger = logging.getLogger("Bcfg2.Client.Frame")
-        if self.setup['drivers']:
-            tools = []
-            for tcandidate in self.setup['drivers'].split(','):
-                if tcandidate not in Bcfg2.Client.Tools.drivers:
-                    self.logger.error("Tool driver %s is not available"\
-                                      % (tcandidate))
-                else:
-                    tools.append(tcandidate)
-        else:
-            tools = Bcfg2.Client.Tools.default[:]
+        for driver in drivers[:]:
+            if driver not in Bcfg2.Client.Tools.drivers and \
+                   isinstance(driver, types.StringType):
+                self.logger.error("Tool driver %s is not available" % driver)
+                drivers.remove(driver)
+                
         tclass = {}
-        for tool in tools:
+        for tool in drivers:
+            if not isinstance(tool, types.StringType):
+                tclass[time.time()] = tool
             tool_class = "Bcfg2.Client.Tools.%s" % tool
             try:
                 tclass[tool] = getattr(__import__(tool_class, globals(),
@@ -69,11 +71,10 @@ class Frame:
             except:
                 self.logger.error("Tool %s unexpectedly failed to load" % tool,
                                   exc_info=1)
-                                  
 
         for tool in tclass.values():
             try:
-                self.tools.append(tool(self.logger, setup, config, self.states))
+                self.tools.append(tool(self.logger, setup, config))
             except Bcfg2.Client.Tools.toolInstantiationError:
                 continue
             except:
@@ -86,13 +87,16 @@ class Frame:
 
         self.logger.info("Loaded tool drivers:")
         self.logger.info([tool.__name__ for tool in self.tools])
-        if not self.setup['dryrun']:
+        if not self.dryrun and not self.setup['bundle']:
             for cfile in [cfl for cfl in config.findall(".//ConfigFile") \
                           if cfl.get('name') in self.__important__]:
                 tl= [t for t in self.tools if t.handlesEntry(cfile) \
                      and t.canVerify(cfile)]
                 if tl:
                     if not tl[0].VerifyConfigFile(cfile, []):
+                        if self.setup['interactive'] and not \
+                               promptFilter("Install %s: %s? (y/N):", [cfile]): 
+                            continue 
                         try:
                             self.states[cfile] = tl[0].InstallConfigFile(cfile)
                         except:
@@ -100,12 +104,6 @@ class Frame:
                                               exc_info=1)
         # find entries not handled by any tools
         problems = [entry for struct in config for entry in struct if entry not in self.handled]
-        if toolset_defs.has_key(config.get('toolset')):
-            tdefs = toolset_defs[config.get('toolset')]
-            for problem in problems[:]:
-                if tdefs.has_key(problem.tag):
-                    problem.set('type', tdefs[problem.tag])
-                    problems.remove(problem)
 
         if problems:
             self.logger.error("The following entries are not handled by any tool:")
@@ -121,6 +119,7 @@ class Frame:
         if multi:
             self.logger.debug("The following entries are included multiple times:")
             self.logger.debug(["%s:%s" % entry for entry in multi])
+            self.logger.debug("")
             
                     
     def __getattr__(self, name):
@@ -141,7 +140,7 @@ class Frame:
                 self.states[entry] = False
         for tool in self.tools:
             try:
-                tool.Inventory()
+                tool.Inventory(self.states)
             except:
                 self.logger.error("%s.Inventory() call failed:" % tool.__name__, exc_info=1)
 
@@ -159,7 +158,25 @@ class Frame:
 
         candidates = [entry for entry in self.states if not self.states[entry]]
         self.whitelist = [entry for entry in self.states if not self.states[entry]]
-        if self.setup['dryrun']:
+        # Need to process decision stuff early, so that dryrun mode works with it
+        if self.setup['decision'] == 'whitelist':
+            dwl = self.setup['decision_list']
+            w_to_rem = [e for e in self.whitelist \
+                        if not matches_white_list(e, dwl)]
+            if w_to_rem:
+                self.logger.info("In whitelist mode: suppressing installation of:")
+                self.logger.info(["%s:%s" % (e.tag, e.get('name')) for e in w_to_rem])
+                self.whitelist = [x for x in self.whitelist if x not in w_to_rem]
+
+        elif self.setup['decision'] == 'blacklist':
+            b_to_rem = [e for e in self.whitelist \
+                        if not passes_black_list(e, self.setup['decision_list'])]
+            if b_to_rem:
+                self.logger.info("In blacklist mode: suppressing installation of:")
+                self.logger.info(["%s:%s" % (e.tag, e.get('name')) for e in b_to_rem])
+                self.whitelist = [x for x in self.whitelist if x not in b_to_rem]
+            
+        if self.dryrun:
             if self.whitelist:
                 self.logger.info("In dryrun mode: suppressing entry installation for:")
                 self.logger.info(["%s:%s" % (entry.tag, entry.get('name')) for entry \
@@ -175,7 +192,7 @@ class Frame:
         # first perform bundle filtering
         if self.setup['bundle']:
             bundles = [b for b in self.config.findall('./Bundle') \
-                       if b.get('name') == self.setup['bundle']]
+                       if b.get('name') in self.setup['bundle']]
             self.whitelist = [e for e in self.whitelist if \
                               True in [e in b for b in bundles]]
         else:
@@ -220,7 +237,7 @@ class Frame:
             if not handled:
                 continue
             try:
-                tool.Install(handled)
+                tool.Install(handled, self.states)
             except:
                 self.logger.error("%s.Install() call failed:" % tool.__name__, exc_info=1)
 
@@ -240,7 +257,7 @@ class Frame:
             tbm = [(t, b) for t in self.tools for b in mbundles]
             for tool, bundle in tbm:
                 try:
-                    tool.Inventory([bundle])
+                    tool.Inventory(self.states, [bundle])
                 except:
                     self.logger.error("%s.Inventory() call failed:" % tool.__name__, exc_info=1)
             clobbered = [entry for bundle in mbundles for entry in bundle \
@@ -256,9 +273,9 @@ class Frame:
             for tool in self.tools:
                 try:
                     if bundle in mbundles:
-                        tool.BundleUpdated(bundle)
+                        tool.BundleUpdated(bundle, self.states)
                     else:
-                        tool.BundleNotUpdated(bundle)
+                        tool.BundleNotUpdated(bundle, self.states)
                 except:
                     self.logger.error("%s.BundleNotUpdated() call failed:" % \
                                       (tool.__name__), exc_info=1)
@@ -294,7 +311,7 @@ class Frame:
 
     def ReInventory(self):
         '''Recheck everything'''
-        if not self.setup['dryrun'] and self.setup['kevlar']:
+        if not self.dryrun and self.setup['kevlar']:
             self.logger.info("Rechecking system inventory")
             self.Inventory()
 
