@@ -1,5 +1,5 @@
 '''This is the bcfg2 support for apt-get'''
-__revision__ = '$Revision: 3909 $'
+__revision__ = '$Revision: 4954 $'
 
 import apt_pkg
 import os, re
@@ -10,23 +10,29 @@ class APT(Bcfg2.Client.Tools.PkgTool):
     the rest from Toolset.Toolset'''
     __name__ = 'APT'
     __execs__ = ['/usr/bin/debsums', '/usr/bin/apt-get', '/usr/bin/dpkg']
-    __important__ = ["/etc/apt/sources.list", "/var/cache/debconf/config.dat", \
-                     "/var/cache/debconf/templates.dat", '/etc/passwd', '/etc/group', \
+    __important__ = ["/etc/apt/sources.list",
+                     "/var/cache/debconf/config.dat", 
+                     "/var/cache/debconf/templates.dat",
+                     '/etc/passwd', '/etc/group', 
                      '/etc/apt/apt.conf', '/etc/dpkg/dpkg.cfg']
     __handles__ = [('Package', 'deb')]
     __req__ = {'Package': ['name', 'version']}
     pkgtype = 'deb'
-    pkgtool = ('apt-get --reinstall -q=2 --force-yes -y install %s',
-               ('%s=%s', ['name', 'version']))
+    pkgtool = ('apt-get '
+               '-o DPkg::Options::=--force-overwrite '
+               '-o DPkg::Options::=--force-confold '
+               '--reinstall '
+               '-q=2 '
+               '--force-yes '
+               '-y install %s',
+              ('%s=%s', ['name', 'version']))
     
     svcre = re.compile("/etc/.*/[SK]\d\d(?P<name>\S+)")
 
-    def __init__(self, logger, cfg, setup, states):
-        Bcfg2.Client.Tools.PkgTool.__init__(self, logger, cfg, setup, states)
+    def __init__(self, logger, cfg, setup):
+        Bcfg2.Client.Tools.PkgTool.__init__(self, logger, cfg, setup)
         self.cfg = cfg
         os.environ["DEBIAN_FRONTEND"] = 'noninteractive'
-        self.installed = {}
-        self.RefreshPackages()
         self.updated = False
 
     def RefreshPackages(self):
@@ -68,13 +74,19 @@ class APT(Bcfg2.Client.Tools.PkgTool):
                             self.logger.error("Got Unsupported pattern %s from debsums" \
                                               % item)
                             files.append(item)
+                    # We check if there is file in the checksum to do
+                    if files:
+                        # if files are found there we try to be sure our modlist is sane
+                        # with erroneous symlinks
+                        modlist = [os.path.realpath(filename) for filename in modlist]
                     bad = [filename for filename in files if filename not in modlist]
                     if bad:
                         self.logger.info("Package %s failed validation. Bad files are:" % \
                                          entry.get('name'))
                         self.logger.info(bad)
                         entry.set('qtext',
-                                  "Reinstall Package %s-%s to fix failing md5sums? (y/N) ")
+                                  "Reinstall Package %s-%s to fix failing md5sums? (y/N) " % \
+                                 (entry.get('name'), entry.get('version')))
                         return False
                 return True
             else:
@@ -93,15 +105,15 @@ class APT(Bcfg2.Client.Tools.PkgTool):
         if len(packages) > 0:
             self.logger.info('Removing packages:')
             self.logger.info(pkgnames)
-            if self.cmd.run("apt-get remove -y --force-yes %s" % pkgnames)[0] == 0:
+            if self.cmd.run("apt-get remove --purge -y --force-yes %s" % pkgnames)[0] == 0:
                 self.modified += packages
             self.RefreshPackages()
             self.extra = self.FindExtraPackages()
               
-    def Install(self, packages):
+    def Install(self, packages, states):
         if self.setup['kevlar'] and not self.setup['dryrun'] and not self.updated:
             self.cmd.run("dpkg --force-confold --configure --pending")
             self.cmd.run("apt-get clean")
             self.cmd.run("apt-get -q=2 -y update")
             self.updated = True
-        Bcfg2.Client.Tools.PkgTool.Install(self, packages)
+        Bcfg2.Client.Tools.PkgTool.Install(self, packages, states)
