@@ -1,8 +1,8 @@
 # This is the bcfg2 support for chkconfig
-# $Id: Chkconfig.py 4237 2008-01-11 03:31:31Z desai $
+# $Id: Chkconfig.py 4940 2008-10-10 02:13:25Z desai $
 
 '''This is chkconfig support'''
-__revision__ = '$Revision: 4237 $'
+__revision__ = '$Revision: 4940 $'
 
 import Bcfg2.Client.Tools, Bcfg2.Client.XML
 
@@ -25,8 +25,13 @@ class Chkconfig(Bcfg2.Client.Tools.SvcTool):
             # Ocurrs when no lines are returned (service not installed)
             entry.set('current_status', 'off')
             return False
-        if entry.get('type') == 'xinetd':
-            return entry.get('status') == srvdata[1]
+        if len(srvdata) == 2:
+            # This is an xinetd service
+            if entry.get('status') == srvdata[1]:
+                return True
+            else:
+                entry.set('current_status', srvdata[1])
+                return False
 
         try:
             onlevels = [level.split(':')[0] for level in srvdata[1:] \
@@ -40,6 +45,10 @@ class Chkconfig(Bcfg2.Client.Tools.SvcTool):
             status = (len(onlevels) == 0)
 
         if entry.get('supervised', 'false') == 'true':
+            pstatus, pout = self.cmd.run('/sbin/service %s status' % \
+                                         entry.get('name'))
+            if pstatus:
+                self.cmd.run('/sbin/service %s start' % (entry.get('name')))
             pstatus, pout = self.cmd.run('/sbin/service %s status' % \
                                          entry.get('name'))
             # chkconfig/init.d service
