@@ -1,5 +1,5 @@
 '''All POSIX Type client support for Bcfg2'''
-__revision__ = '$Revision: 4292 $'
+__revision__ = '$Revision: 4923 $'
 
 from stat import S_ISVTX, S_ISGID, S_ISUID, S_IXUSR, S_IWUSR, S_IRUSR, S_IXGRP
 from stat import S_IWGRP, S_IRGRP, S_IXOTH, S_IWOTH, S_IROTH, ST_MODE, S_ISDIR
@@ -89,9 +89,13 @@ class POSIX(Bcfg2.Client.Tools.Tool):
             self.logger.debug("Symlink %s points to %s, should be %s" % \
                               (entry.get('name'), sloc, entry.get('to')))
             entry.set('current_to', sloc)
+            entry.set('qtext', "Link %s to %s? [y/N] " % (entry.get('name'),
+                                                   entry.get('to')))
             return False
         except OSError:
             entry.set('current_exists', 'false')
+            entry.set('qtext', "Link %s to %s? [y/N] " % (entry.get('name'),
+                                                   entry.get('to')))
             return False
 
     def InstallSymLink(self, entry):
@@ -146,24 +150,24 @@ class POSIX(Bcfg2.Client.Tools.Tool):
                  (mtime == entry.get('mtime', '-1')))
 
         pruneTrue = True
+        ex_ents = []
         if entry.get('prune', 'false') == 'true' \
                and entry.tag == 'Directory':
             try:
                 entries = ['/'.join([entry.get('name'), ent]) \
                            for ent in os.listdir(entry.get('name'))]
-                self.ex_ents = [e for e in entries if e not in modlist]
-                if self.ex_ents:
+                ex_ents = [e for e in entries if e not in modlist]
+                if ex_ents:
                     pruneTrue = False
                     self.logger.debug("Directory %s contains extra entries:" % entry.get('name'))
-                    self.logger.debug(self.ex_ents)
+                    self.logger.debug(ex_ents)
                     nqtext = entry.get('qtext', '') + '\n'
                     nqtext += "Directory %s contains extra entries:" % entry.get('name')
-                    nqtext += ":".join(self.ex_ents)
+                    nqtext += ":".join(ex_ents)
                     entry.set('qtest', nqtext)
             except OSError:
+                ex_ents = []
                 pruneTrue = True
-        else:
-            pruneTrue = True
 
         if not pTrue:
             if owner != str(normUid(entry)):
@@ -213,6 +217,7 @@ class POSIX(Bcfg2.Client.Tools.Tool):
                 self.logger.debug("Found a non-directory entry at %s" % (entry.get('name')))
                 try:
                     os.unlink(entry.get('name'))
+                    exists = False
                 except OSError:
                     self.logger.info("Failed to unlink %s" % (entry.get('name')))
                     return False
@@ -252,8 +257,8 @@ class POSIX(Bcfg2.Client.Tools.Tool):
             except OSError:
                 self.logger.error('Failed to create directory %s' % (entry.get('name')))
                 return False
-        if entry.get('prune', 'false') == 'true' and self.ex_ents:
-            for pname in self.ex_ents:
+        if entry.get('prune', 'false') == 'true' and entry.get("qtest"):
+            for pname in entry.get("qtest").split(":"):
                 ulfailed = False
                 try:
                     self.logger.debug("Unlinking file %s" % pname)
@@ -315,11 +320,15 @@ class POSIX(Bcfg2.Client.Tools.Tool):
                 self.logger.error("Cannot verify incomplete ConfigFile %s" % (entry.get('name')))
                 return False
             tempdata = entry.text
+            if type(tempdata) == unicode:
+                tempdata = tempdata.encode(self.setup['encoding'])
         try:
             content = open(entry.get('name')).read()
         except IOError, error:
             self.logger.error("Failed to read %s: %s" % (error.filename, error.strerror))
             return False
+        # comparison should be done with fingerprints or md5sum so it would be faster
+        # for big binary files
         contentStatus = content == tempdata
         if not contentStatus:
             if tbin or not isString(content):
@@ -344,6 +353,7 @@ class POSIX(Bcfg2.Client.Tools.Tool):
                 if do_diff:
                     diff = '\n'.join(rawdiff)
                     entry.set("current_bdiff", binascii.b2a_base64(diff))
+#                    entry.set("current_diff", diff)
                     udiff = '\n'.join([x for x in \
                                        difflib.unified_diff(content.split('\n'), \
                                                             tempdata.split('\n'))])
@@ -351,6 +361,7 @@ class POSIX(Bcfg2.Client.Tools.Tool):
                         eudiff = udiff.encode('ascii')
                     except:
                         eudiff = "Binary file: no diff printed"
+ 
                     nqtext = entry.get('qtext', '')
 
                     if nqtext:
@@ -398,7 +409,8 @@ class POSIX(Bcfg2.Client.Tools.Tool):
                             return False
 
         # If we get here, then the parent directory should exist
-        if entry.get("paranoid", False) and self.setup.get("paranoid", False):
+        if entry.get("paranoid", False) and self.setup.get("paranoid", False) \
+               and not (entry.get('current_exists', 'true') == 'false'):
             bkupnam = entry.get('name').replace('/', '_')
             if self.cmd.run("cp %s /var/cache/bcfg2/%s" % (entry.get('name'), bkupnam))[0]:
                 self.logger.error("Failed to create backup file for ConfigFile %s" % \
@@ -411,7 +423,10 @@ class POSIX(Bcfg2.Client.Tools.Tool):
             elif entry.get('empty', 'false') == 'true':
                 filedata = ''
             else:
-                filedata = entry.text
+                if type(entry.text) == unicode:
+                    filedata = entry.text.encode(self.setup['encoding'])
+                else:
+                    filedata = entry.text
             newfile.write(filedata)
             newfile.close()
             try:
