@@ -1,5 +1,5 @@
 '''This is the bcfg2 tool for the Gentoo Portage system.'''
-__revision__ = '$Revision: 2870 $'
+__revision__ = '$Revision: 4923 $'
 
 import re
 import Bcfg2.Client.Tools
@@ -16,8 +16,8 @@ class Portage(Bcfg2.Client.Tools.PkgTool):
     # requires a working PORTAGE_BINHOST in make.conf
     pkgtool = ('emerge --getbinpkgonly %s', ('=%s-%s', ['name', 'version']))
     
-    def __init__(self, logger, cfg, setup, states):
-        Bcfg2.Client.Tools.PkgTool.__init__(self, logger, cfg, setup, states)
+    def __init__(self, logger, cfg, setup):
+        Bcfg2.Client.Tools.PkgTool.__init__(self, logger, cfg, setup)
         self.cfg = cfg
         self.installed = {}
         self.RefreshPackages()
@@ -42,12 +42,12 @@ class Portage(Bcfg2.Client.Tools.PkgTool):
         if self.installed.has_key(entry.attrib['name']):
             if self.installed[entry.attrib['name']] == entry.attrib['version']:
                 if not self.setup['quick'] and \
-				entry.get('verify', 'true') == 'true':
+                                entry.get('verify', 'true') == 'true':
                     output = self.cmd.run \
-				    ("/usr/bin/equery check =%s 2>&1 |grep '!!!' \
-					| awk '{print $2}'" % entry.get('name'))[1]
+                                  ("/usr/bin/equery check =%s 2>&1 |grep '!!!' \
+                                    | awk '{print $2}'" % entry.get('name'))[1]
                     if [filename for filename in output \
-				    if filename not in modlist]:
+                                    if filename not in modlist]:
                         return False
                 return True
             else:
