@@ -1,6 +1,6 @@
 '''Bcfg2 Support for RPMS'''
 
-__revision__ = '$Revision: 3656 $'
+__revision__ = '$Revision: 4923 $'
 
 import Bcfg2.Client.Tools, rpmtools, os.path, rpm, ConfigParser, sys
 
@@ -34,8 +34,8 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
     pkgtype = 'rpm'
     pkgtool = ("rpm --oldpackage --replacepkgs --quiet -U %s", ("%s", ["url"]))
 
-    def __init__(self, logger, setup, config, states):
-        Bcfg2.Client.Tools.PkgTool.__init__(self, logger, setup, config, states)
+    def __init__(self, logger, setup, config):
+        Bcfg2.Client.Tools.PkgTool.__init__(self, logger, setup, config)
 
         self.instance_status = {}
         self.extra_instances = []
@@ -108,6 +108,16 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
             self.verify_fail_action = 'none'
         self.logger.debug('verify_fail_action = %s' % self.verify_fail_action)
 
+        # version_fail_action
+        if RPMng_CP.has_option(self.__name__, 'verify_flags'):
+            self.verify_flags = RPMng_CP.get(self.__name__, 'verify_flags').lower().split(',')
+        else:
+            self.verify_flags = []
+        if '' in self.verify_flags:
+            self.verify_flags.remove('')
+        self.logger.debug('version_fail_action = %s' % self.version_fail_action)
+
+
     def RefreshPackages(self):
         '''
             Creates self.installed{} which is a dict of installed packages.
@@ -202,7 +212,7 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
          
                                 if self.pkg_verify == 'true' and \
                                    inst.get('pkg_verify', 'true') == 'true':
-                                    flags = inst.get('verify_flags', '').split(',')
+                                    flags = inst.get('verify_flags', '').split(',') + self.verify_flags
                                     if pkg.get('gpgkeyid', '')[-8:] not in self.gpg_keyids and \
                                        entry.get('name') != 'gpg-pubkey':
                                         flags += ['nosignature', 'nodigest']
@@ -257,7 +267,7 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
     
                                     if self.pkg_verify == 'true' and \
                                        inst.get('pkg_verify', 'true') == 'true':
-                                        flags = inst.get('verify_flags', '').split(',') 
+                                        flags = inst.get('verify_flags', '').split(',') + self.verify_flags
                                         if pkg.get('gpgkeyid', '')[-8:] not in self.gpg_keyids:
                                             flags += ['nosignature', 'nodigest']
                                             self.logger.info('WARNING: Package %s %s requires GPG Public key with ID %s'\
@@ -524,7 +534,7 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
 
         return fix
     
-    def Install(self, packages):
+    def Install(self, packages, states):
         '''
            Try and fix everything that RPMng.VerifyPackages() found wrong for 
            each Package Entry.  This can result in individual RPMs being 
@@ -532,14 +542,14 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
            or upgraded.
 
            packages is a list of Package Elements that has 
-               self.states[<Package Element>] == False
+               states[<Package Element>] == False
 
            The following effects occur:
-           - self.states{} is conditionally updated for each package.
+           - states{} is conditionally updated for each package.
            - self.installed{} is rebuilt, possibly multiple times.
            - self.instance_statusi{} is conditionally updated for each instance 
              of a package.
-           - Each package will be added to self.modified[] if its self.states{} 
+           - Each package will be added to self.modified[] if its states{} 
              entry is set to True. 
         '''
         self.logger.info('Runing RPMng.Install()')
@@ -628,7 +638,7 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
             self.RefreshPackages()
             self.gpg_keyids = self.getinstalledgpg()
             pkg = self.instance_status[gpg_keys[0]].get('pkg')
-            self.states[pkg] = self.VerifyPackage(pkg, [])
+            states[pkg] = self.VerifyPackage(pkg, [])
 
         # Fix upgradeable packages.
         if len(upgrade_pkgs) > 0:
@@ -669,10 +679,10 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
         if not self.setup['kevlar']:
             for pkg_entry in packages:
                 self.logger.debug("Reverifying Failed Package %s" % (pkg_entry.get('name')))
-                self.states[pkg_entry] = self.VerifyPackage(pkg_entry, \
-                                                                 self.modlists.get(pkg_entry, []))
+                states[pkg_entry] = self.VerifyPackage(pkg_entry, \
+                                                       self.modlists.get(pkg_entry, []))
 
-        for entry in [ent for ent in packages if self.states[ent]]:
+        for entry in [ent for ent in packages if states[ent]]:
             self.modified.append(entry)
 
     def canInstall(self, entry):
@@ -759,7 +769,7 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
             return False
 
         # We don't want to do any checks so we don't care what the entry has in it.
-        if self.pkg_checks == 'false' or entry.get('pkg_checks', 'true') == 'false':
+        if self.pkg_checks == 'false' or entry.get('pkg_checks', 'true').lower() == 'false':
             return True
 
         instances = entry.findall('Instance')
@@ -900,7 +910,7 @@ class RPMng(Bcfg2.Client.Tools.PkgTool):
         else: 
             return '%s:%s-%s.%s' % (instance.get('epoch', '*'),
                                     instance.get('version', '*'), 
- 	                            instance.get('release', '*'),
+                                    instance.get('release', '*'),
                                     instance.get('arch', '*')) 
 
     def pkg_vr_equal(self, config_entry, installed_entry):
