@@ -1,6 +1,6 @@
 # This is the bcfg2 support for solaris sysv packages
 '''This provides bcfg2 support for Solaris SYSV packages'''
-__revision__ = '$Revision: 4210 $'
+__revision__ = '$Revision: 4703 $'
 
 import tempfile, Bcfg2.Client.Tools, Bcfg2.Client.XML
 
@@ -21,7 +21,7 @@ basedir=default
 
 class SYSV(Bcfg2.Client.Tools.PkgTool):
     '''Solaris SYSV package support'''
-    __execs__ = ["/usr/sbin/pkgadd"]
+    __execs__ = ["/usr/sbin/pkgadd", "/usr/bin/pkginfo"]
     __handles__ = [('Package', 'sysv')]
     __req__ = {'Package': ['name', 'version']}
     __ireq__ = {'Package': ['name', 'url', 'version']}
@@ -29,8 +29,8 @@ class SYSV(Bcfg2.Client.Tools.PkgTool):
     pkgtype = 'sysv'
     pkgtool = ("/usr/sbin/pkgadd %s -n -d %%s", (('%s %s', ['url', 'name'])))
 
-    def __init__(self, logger, setup, config, states):
-        Bcfg2.Client.Tools.PkgTool.__init__(self, logger, setup, config, states)
+    def __init__(self, logger, setup, config):
+        Bcfg2.Client.Tools.PkgTool.__init__(self, logger, setup, config)
         self.noaskname = tempfile.mktemp()
         try:
             open(self.noaskname, 'w+').write(noask)
