@@ -1,21 +1,21 @@
 '''This module sets up a base list of configuration entries'''
-__revision__ = '$Revision: 2695 $'
+__revision__ = '$Revision: 4733 $'
 
 import Bcfg2.Server.Plugin
 import copy
 import lxml.etree
 
-class Base(Bcfg2.Server.Plugin.Plugin, Bcfg2.Server.Plugin.XMLDirectoryBacked):
+class Base(Bcfg2.Server.Plugin.StructurePlugin,
+           Bcfg2.Server.Plugin.XMLDirectoryBacked):
     '''This Structure is good for the pile of independent configs needed for most actual systems'''
     __name__ =  'Base'
-    __version__ = '$Id: Base.py 2695 2007-01-18 16:41:20Z desai $'
+    __version__ = '$Id: Base.py 4733 2008-07-01 18:05:08Z desai $'
     __author__ = 'bcfg-dev@mcs.anl.gov'
     __child__ = Bcfg2.Server.Plugin.StructFile
     
     '''base creates independent clauses based on client metadata'''
     def __init__(self, core, datastore):
         Bcfg2.Server.Plugin.Plugin.__init__(self, core, datastore)
-        self.fragements = {}
         try:
             Bcfg2.Server.Plugin.XMLDirectoryBacked.__init__(self, self.data, self.core.fam)
         except OSError:
