@@ -1,12 +1,13 @@
 '''This provides bundle clauses with translation functionality'''
-__revision__ = '$Revision: 2695 $'
+__revision__ = '$Revision: 4680 $'
 
 import copy, lxml.etree, Bcfg2.Server.Plugin
 
-class Bundler(Bcfg2.Server.Plugin.Plugin, Bcfg2.Server.Plugin.XMLDirectoryBacked):
+class Bundler(Bcfg2.Server.Plugin.StructurePlugin,
+              Bcfg2.Server.Plugin.XMLDirectoryBacked):
     '''The bundler creates dependent clauses based on the bundle/translation scheme from bcfg1'''
     __name__ =  'Bundler'
-    __version__ = '$Id: Bundler.py 2695 2007-01-18 16:41:20Z desai $'
+    __version__ = '$Id: Bundler.py 4680 2008-06-09 15:56:41Z desai $'
     __author__ = 'bcfg-dev@mcs.anl.gov'
     __child__ = Bcfg2.Server.Plugin.StructFile
     
