@@ -1,11 +1,11 @@
 '''This file provides the Hostbase plugin. It manages dns/dhcp/nis host information'''
-__revision__ = '$Revision: 3906 $'
+__revision__ = '$Revision: 4949 $'
 
 import sys, os
 os.environ['DJANGO_SETTINGS_MODULE'] = 'Bcfg2.Server.Hostbase.settings'
 from lxml.etree import Element, SubElement
 from syslog import syslog, LOG_INFO
-from Bcfg2.Server.Plugin import Plugin, PluginExecutionError, PluginInitError, DirectoryBacked
+from Bcfg2.Server.Plugin import StructurePlugin, PluginExecutionError, PluginInitError, DirectoryBacked, GeneratorPlugin
 from time import strftime
 from sets import Set
 from django.template import Context, loader
@@ -37,17 +37,17 @@ import cStringIO
 ##         '''This function is called when underlying data has changed'''
 ##         pass
     
-class Hostbase(Plugin):
+class Hostbase(StructurePlugin,GeneratorPlugin):
     '''The Hostbase plugin handles host/network info'''
     __name__ = 'Hostbase'
-    __version__ = '$Id: Hostbase.py 3906 2007-11-02 23:30:39Z desai $'
+    __version__ = '$Id: Hostbase.py 4949 2008-10-16 18:42:00Z bradshaw $'
     __author__ = 'bcfg-dev@mcs.anl.gov'
     filepath = '/my/adm/hostbase/files/bind'
 
     def __init__(self, core, datastore):
 
         self.ready = False
-        Plugin.__init__(self, core, datastore)
+        StructurePlugin.__init__(self, core, datastore)
         files = ['zone.tmpl', 'reversesoa.tmpl', 'named.tmpl', 'reverseappend.tmpl',
                  'dhcpd.tmpl', 'hosts.tmpl', 'hostsappend.tmpl']
 ##         try:
@@ -255,6 +255,9 @@ class Hostbase(Plugin):
 
             self.filedata[reversezone[1]] = self.templates['reversesoa'].render(context)
             self.filedata[reversezone[1] + '.external'] = self.templates['reversesoa'].render(context)
+            self.filedata[reversezone[1]] += reversezone[9]
+            self.filedata[reversezone[1] + '.external'] += reversezone[9]
+
             subnet = reversezone[1].split(".")
             subnet.reverse()
             reversenames.append((reversezone[1].rstrip('.rev'),".".join(subnet[1:]))) 
@@ -355,7 +358,7 @@ class Hostbase(Plugin):
         hosts = []
         hostdata = [dhcphosts[0][0], dhcphosts[0][1], dhcphosts[0][2]]
         if len(dhcphosts) > 1:
-            for x in range(1, len(cursor.fetchall())):
+            for x in range(1, len(dhcphosts)):
                 # if an interface has 2 or more ip addresses
                 # adds the ip to the current interface
                 if hostdata[0].split(".")[0] == dhcphosts[x][0].split(".")[0] and hostdata[1] == dhcphosts[x][1]:
