@@ -1,7 +1,7 @@
-'''This file stores persistent metadata for the BCFG Configuration Repository'''
-__revision__ = '$Revision: 3948 $'
+'''This file stores persistent metadata for the Bcfg2 Configuration Repository'''
+__revision__ = '$Revision: 4942 $'
 
-import lxml.etree, re, socket, time, sys, ConfigParser
+import lxml.etree, re, socket, time, fcntl, copy
 import Bcfg2.Server.Plugin
 
 class MetadataConsistencyError(Exception):
@@ -12,34 +12,109 @@ class MetadataRuntimeError(Exception):
     '''This error is raised when the metadata engine is called prior to reading enough data'''
     pass
 
+probe_matcher = re.compile("(?P<basename>\S+)(.(?P<mode>[GH])_\S+)?")
+
 class ClientMetadata(object):
     '''This object contains client metadata'''
-    def __init__(self, client, groups, bundles, toolset, categories, probed, uuid, password):
+    def __init__(self, client, groups, bundles, categories, probed, uuid,
+                 password, overall):
         self.hostname = client
         self.bundles = bundles
         self.groups = groups
-        self.toolset = toolset
         self.categories = categories
         self.probes = probed
         self.uuid = uuid
         self.password = password
+        self.all = overall
 
     def inGroup(self, group):
         '''Test to see if client is a member of group'''
         return group in self.groups
 
-class Metadata(Bcfg2.Server.Plugin.Plugin):
+    def get_clients_by_group(self, group):
+        """
+        return a list of clients that are members of a group
+        Arguments:
+        - `group`: group name
+        """
+        profiles = [key for key, value in self.all[0].iteritems() \
+                    if group in value[1]]
+        return [key for key, value in self.all[1].iteritems() \
+                if value in profiles]
+
+    def get_clients_by_profile(self, profile):
+        """
+        return clients with a given profile
+        Arguments:
+        - `profile`: profile name
+        """
+        return [key for key, value in self.all[1].iteritems() \
+                if value == profile]
+
+class ProbeSet(Bcfg2.Server.Plugin.EntrySet):
+    def __init__(self, path, fam, encoding, plugin_name):
+        fpattern = '[0-9A-Za-z_\-]+'
+        self.plugin_name = plugin_name
+        Bcfg2.Server.Plugin.EntrySet.__init__(self, fpattern, path, True, 
+                                              Bcfg2.Server.Plugin.SpecificData,
+                                              encoding)
+        fam.AddMonitor(path, self)
+        self.bangline = re.compile('^#!(?P<interpreter>(/\w+)+)$')
+
+    def HandleEvent(self, event):
+        if event.filename != self.path:
+            return self.handle_event(event)
+
+    def get_probe_data(self, metadata):
+        ret = []
+        candidates = self.get_matching(metadata)
+        temp = {}
+        for cand in candidates:
+            if cand.specific.all:
+                if cand.name not in temp:
+                    temp[cand.name] = (cand, 0)
+                continue
+            mdata = probe_matcher.match(cand.name).groupdict()
+            if mdata['basename'] in temp:
+                if mdata['mode'] > temp[mdata['basename']][1]:
+                    temp[mdata['basename']] = (cand, mdata['mode'])
+            else:
+                temp[mdata['basename']] = (cand, mdata['mode'])
+        
+        for (name, data) in temp.iteritems():
+            entry, prio = data
+            probe = lxml.etree.Element('probe')
+            probe.set('name', name.split('/')[-1])
+            probe.set('source', self.plugin_name)
+            probe.text = entry.data
+            match = self.bangline.match(entry.data.split('\n')[0])
+            if match:
+                probe.set('interpreter', match.group('interpreter'))
+            else:
+                probe.set('interpreter', '/bin/sh')
+            ret.append(probe)
+        return ret
+
+
+class Metadata(Bcfg2.Server.Plugin.MetadataPlugin,
+               Bcfg2.Server.Plugin.ProbingPlugin):
     '''This class contains data for bcfg2 server metadata'''
-    __version__ = '$Id: Metadata.py 3948 2007-11-12 19:31:22Z desai $'
+    __version__ = '$Id: Metadata.py 4942 2008-10-10 07:20:42Z solj $'
     __author__ = 'bcfg-dev@mcs.anl.gov'
     __name__ = "Metadata"
 
-    def __init__(self, core, datastore):
+    def __init__(self, core, datastore, watch_clients=True):
         Bcfg2.Server.Plugin.Plugin.__init__(self, core, datastore)
-        self.__name__ = 'Metadata'
-        core.fam.AddMonitor("%s/%s" % (self.data, "groups.xml"), self)
-        core.fam.AddMonitor("%s/%s" % (self.data, "clients.xml"), self)
-        self.states = {'groups.xml':False, 'clients.xml':False}
+        #self.__name__ = 'Metadata'
+        if watch_clients:
+            try:
+                core.fam.AddMonitor("%s/%s" % (self.data, "groups.xml"), self)
+                core.fam.AddMonitor("%s/%s" % (self.data, "clients.xml"), self)
+            except:
+                raise Bcfg2.Server.Plugin.PluginInitError
+        self.states = {}
+        if watch_clients:
+            self.states = {"groups.xml":False, "clients.xml":False}
         self.addresses = {}
         self.clients = {}
         self.aliases = {}
@@ -47,7 +122,6 @@ class Metadata(Bcfg2.Server.Plugin.Plugin):
         self.cgroups = {}
         self.public = []
         self.profiles = []
-        self.toolsets = {}
         self.categories = {}
         self.bad_clients = {}
         self.uuid = {}
@@ -56,18 +130,81 @@ class Metadata(Bcfg2.Server.Plugin.Plugin):
         self.passwords = {}
         self.session_cache = {}
         self.clientdata = None
+        self.clientdata_original = None
         self.default = None
         self.pdirty = False
         try:
             loc = datastore + "/Probes"
-            self.probes = Bcfg2.Server.Plugin.DirectoryBacked(loc, core.fam)
+            self.probes = ProbeSet(loc, core.fam, core.encoding, self.__name__)
         except:
             self.probes = False
         self.probedata = {}
-        self.ptimes = {}
-        self.pctime = 0
         self.extra = {'groups.xml':[], 'clients.xml':[]}
-        self.password = core.cfile.get('communication', 'password')
+        self.password = core.password
+        self.load_probedata()
+
+    def get_groups(self):
+        '''return groups xml tree'''
+        groups_tree = lxml.etree.parse(self.data + "/groups.xml")
+        root = groups_tree.getroot()
+        return root
+
+    def search_client(self, client_name, tree):
+        '''find a client'''
+        for node in tree.findall("//Client"):
+            if node.get("name") == client_name:
+                return node
+            for child in node:
+                if child.tag == "Alias" and child.attrib["name"] == client_name:
+                    return node
+        return None
+
+    def add_client(self, client_name, attribs):
+        '''add client to clients.xml'''
+        tree = lxml.etree.parse(self.data + "/clients.xml")
+        root = tree.getroot()
+        element = lxml.etree.Element("Client", name=client_name)
+        for key, val in attribs.iteritems():
+            element.set(key, val)
+        node = self.search_client(client_name, tree)
+        if node != None:
+            self.logger.error("Client \"%s\" already exists" % (client_name))
+            raise MetadataConsistencyError
+        root.append(element)
+        client_tree = open(self.data + "/clients.xml","w")
+        fd = client_tree.fileno()
+        while True:
+            try:
+                fcntl.lockf(fd, fcntl.LOCK_EX | fcntl.LOCK_NB)
+            except IOError:
+                continue
+            else:
+                break
+        tree.write(client_tree)
+        fcntl.lockf(fd, fcntl.LOCK_UN)
+        client_tree.close()
+
+    def remove_client(self, client_name):
+        '''Remove a client'''
+        tree = lxml.etree.parse(self.data + "/clients.xml")
+        root = tree.getroot()
+        node = self.search_client(client_name, tree)
+        if node == None:
+            self.logger.error("Client \"%s\" not found" % (client_name))
+            raise MetadataConsistencyError
+        root.remove(node)
+        client_tree = open(self.data + "/clients.xml","w")
+        fd = client_tree.fileno()
+        while True:
+            try:
+                fcntl.lockf(fd, fcntl.LOCK_EX | fcntl.LOCK_NB)
+            except IOError:
+                continue
+            else:
+                break
+        tree.write(client_tree)
+        fcntl.lockf(fd, fcntl.LOCK_UN)
+        client_tree.close()
 
     def HandleEvent(self, event):
         '''Handle update events for data files'''
@@ -89,6 +226,7 @@ class Metadata(Bcfg2.Server.Plugin.Plugin):
             return
         included = [ent.get('href') for ent in \
                     xdata.findall('./{http://www.w3.org/2001/XInclude}include')]
+        xdata_original = copy.deepcopy(xdata)
         if included:
             for name in included:
                 if name not in self.extra[dest]:
@@ -106,6 +244,7 @@ class Metadata(Bcfg2.Server.Plugin.Plugin):
             self.secure = []
             self.floating = []
             self.addresses = {}
+            self.clientdata_original = xdata_original
             self.clientdata = xdata
             for client in xdata.findall('.//Client'):
                 clname = client.get('name').lower()
@@ -117,7 +256,7 @@ class Metadata(Bcfg2.Server.Plugin.Plugin):
                         self.addresses[caddr] = [clname]
                 if 'uuid' in client.attrib:
                     self.uuid[client.get('uuid')] = clname
-                if 'secure' in client.attrib:
+                if client.get('secure', 'false') == 'true' :
                     self.secure.append(clname)
                 if client.get('location', 'fixed') == 'floating':
                     self.floating.append(clname)
@@ -136,7 +275,6 @@ class Metadata(Bcfg2.Server.Plugin.Plugin):
         elif dest == 'groups.xml':
             self.public = []
             self.profiles = []
-            self.toolsets = {}
             self.groups = {}
             grouptmp = {}
             self.categories = {}
@@ -151,14 +289,13 @@ class Metadata(Bcfg2.Server.Plugin.Plugin):
                     self.profiles.append(group.get('name'))
                 if group.get('public', 'false') == 'true':
                     self.public.append(group.get('name'))
-                if group.attrib.has_key('toolset'):
-                    self.toolsets[group.get('name')] = group.get('toolset')
                 if group.attrib.has_key('category'):
                     self.categories[group.get('name')] = group.get('category')
             for group in grouptmp:
                 # self.groups[group] => (bundles, groups, categories)
                 self.groups[group] = ([], [], {})
                 tocheck = [group]
+                group_cat = self.groups[group][2]
                 while tocheck:
                     now = tocheck.pop()
                     if now not in self.groups[group][1]:
@@ -166,11 +303,17 @@ class Metadata(Bcfg2.Server.Plugin.Plugin):
                     if grouptmp.has_key(now):
                         (bundles, groups) = grouptmp[now]
                         for ggg in [ggg for ggg in groups if ggg not in self.groups[group][1]]:
-                            if not self.categories.has_key(ggg) or not self.groups[group][2].has_key(self.categories[ggg]):
+                            if ggg not in self.categories or \
+                                   self.categories[ggg] not in self.groups[group][2]:
                                 self.groups[group][1].append(ggg)
                                 tocheck.append(ggg)
-                            if self.categories.has_key(ggg):
-                                self.groups[group][2][self.categories[ggg]] = ggg
+                                if self.categories.has_key(ggg):
+                                    group_cat[self.categories[ggg]] = ggg
+                            elif ggg in self.categories:
+                                self.logger.info("Group %s: %s cat-suppressed %s" % \
+                                                 (group,
+                                                  group_cat[self.categories[ggg]],
+                                                  ggg))
                         [self.groups[group][0].append(bund) for bund in bundles
                          if bund not in self.groups[group][0]]
         self.states[dest] = True
@@ -179,7 +322,7 @@ class Metadata(Bcfg2.Server.Plugin.Plugin):
             real = self.groups.keys()
             for client in self.clients.keys():
                 if self.clients[client] not in self.profiles:
-                    self.logger.error("Client %s set as nonexistant or incomplete group %s" \
+                    self.logger.error("Client %s set as nonexistent or incomplete group %s" \
                                       % (client, self.clients[client]))
                     self.logger.error("Removing client mapping for %s" % (client))
                     self.bad_clients[client] = self.clients[client]
@@ -200,17 +343,19 @@ class Metadata(Bcfg2.Server.Plugin.Plugin):
             raise MetadataConsistencyError
         if self.clients.has_key(client):
             self.logger.info("Changing %s group from %s to %s" % (client, self.clients[client], profile))
-            cli = self.clientdata.xpath('/Clients/Client[@name="%s"]' % (client))
+            cli = self.clientdata_original.xpath('.//Client[@name="%s"]' % (client))
             cli[0].set('profile', profile)
         else:
+            self.logger.info("Creating new client: %s, profile %s" % \
+                             (client, profile))
             if self.session_cache.has_key(addresspair):
                 # we are working with a uuid'd client
-                lxml.etree.SubElement(self.clientdata.getroot(),
+                lxml.etree.SubElement(self.clientdata_original.getroot(),
                                       'Client', name=client,
                                       uuid=client, profile=profile,
                                       address=addresspair[0])
             else:
-                lxml.etree.SubElement(self.clientdata.getroot(),
+                lxml.etree.SubElement(self.clientdata_original.getroot(),
                                       'Client', name=client,
                                       profile=profile)
         self.clients[client] = profile
@@ -223,9 +368,21 @@ class Metadata(Bcfg2.Server.Plugin.Plugin):
         except IOError:
             self.logger.error("Failed to write clients.xml")
             raise MetadataRuntimeError
-        datafile.write(lxml.etree.tostring(self.clientdata.getroot()))
+        fd = datafile.fileno()
+        while self.locked(fd) == True:
+            pass
+        datafile.write(lxml.etree.tostring(self.clientdata_original.getroot(),
+                                           pretty_print='true'))
+        fcntl.lockf(fd, fcntl.LOCK_UN)
         datafile.close()
-
+    
+    def locked(self, fd):
+        try:
+            fcntl.lockf(fd, fcntl.LOCK_EX | fcntl.LOCK_NB)
+        except IOError:
+            return True
+        return False
+    
     def write_probedata(self):
         '''write probe data out for use with bcfg2-info'''
         if self.pdirty:
@@ -235,11 +392,14 @@ class Metadata(Bcfg2.Server.Plugin.Plugin):
                 for probe in probed:
                     lxml.etree.SubElement(cx, 'Probe', name=probe,
                                           value=self.probedata[client][probe])
-            data = lxml.etree.tostring(top)
+                for group in self.cgroups[client]:
+                    lxml.etree.SubElement(cx, "Group", name=group)
+            data = lxml.etree.tostring(top, encoding='UTF-8', xml_declaration=True,
+                                       pretty_print='true')
             try:
                 datafile = open("%s/%s" % (self.data, 'probed.xml'), 'w')
             except IOError:
-                self.logger.error("Failed to write clients.xml")
+                self.logger.error("Failed to write probed.xml")
                 raise MetadataRuntimeError
             datafile.write(data)
             self.pdirty = False
@@ -250,27 +410,16 @@ class Metadata(Bcfg2.Server.Plugin.Plugin):
         except:
             self.logger.error("Failed to read file probed.xml")
             return
-        self.probedata={}
+        self.probedata = {}
+        self.cgroups = {}
         for client in data.getchildren():
             self.probedata[client.get('name')] = {}
+            self.cgroups[client.get('name')]=[]
             for pdata in client:
-                self.probedata[client.get('name')][pdata.get('name')] = pdata.get('value')
-
-    def find_toolset(self, client):
-        '''Find the toolset for a given client'''
-        tgroups = [self.toolsets[group] for group in self.groups[client][1] if self.toolsets.has_key(group)]
-        if len(tgroups) == 1:
-            return tgroups[0]
-        elif len(tgroups) == 0:
-            self.logger.error("Couldn't find toolset for client %s" % (client))
-            raise MetadataConsistencyError
-        else:
-            self.logger.error("Got goofy toolset result for client %s" % (client))
-            raise MetadataConsistencyError
-
-    def get_config_template(self, client):
-        '''Build the configuration header for a client configuration'''
-        return lxml.etree.Element("Configuration", version='2.0', toolset=self.find_toolset(client))
+                if (pdata.tag == 'Probe'):
+                    self.probedata[client.get('name')][pdata.get('name')] = pdata.get('value')
+                elif (pdata.tag == 'Group'):
+                    self.cgroups[client.get('name')].append(pdata.get('name'))
 
     def resolve_client(self, addresspair):
         '''Lookup address locally or in DNS to get a hostname'''
@@ -286,7 +435,10 @@ class Metadata(Bcfg2.Server.Plugin.Plugin):
                 raise MetadataConsistencyError
             return self.addresses[address][0]
         try:
-            return socket.gethostbyaddr(address)[0].lower()
+            cname = socket.gethostbyaddr(address)[0].lower()
+            if cname in self.aliases:
+                return self.aliases[cname]
+            return cname
         except socket.herror:
             warning = "address resolution error for %s" % (address)
             self.logger.warning(warning)
@@ -305,13 +457,6 @@ class Metadata(Bcfg2.Server.Plugin.Plugin):
                 raise MetadataConsistencyError
             self.set_profile(client, self.default, (None, None))
             [bundles, groups, categories] = self.groups[self.default]
-        toolinfo = [self.toolsets[group] for group in groups if self.toolsets.has_key(group)]
-        if len(toolinfo) > 1:
-            self.logger.error("Found multiple toolsets for client %s; choosing one" % (client))
-        elif len(toolinfo) == 0:
-            self.logger.error("Cannot determine toolset for client %s" % (client))
-            raise MetadataConsistencyError
-        toolset = toolinfo[0]
         probed = self.probedata.get(client, {})
         newgroups = groups[:]
         newbundles = bundles[:]
@@ -334,37 +479,36 @@ class Metadata(Bcfg2.Server.Plugin.Plugin):
             [newbundles.append(b) for b in nbundles if b not in newbundles]
             [newgroups.append(g) for g in ngroups if g not in newgroups]
             newcategories.update(ncategories)
-        return ClientMetadata(client, newgroups, newbundles, toolset, newcategories,
-                              probed, uuid, password)
+        groupscopy = copy.deepcopy(self.groups)
+        clientscopy = copy.deepcopy(self.clients)
+        return ClientMetadata(client, newgroups, newbundles, newcategories,
+                              probed, uuid, password, (groupscopy, clientscopy))
         
     def GetProbes(self, meta, force=False):
         '''Return a set of probes for execution on client'''
-        ret = []
-        ctime = time.time() - self.ptimes.get(meta.hostname, 0)
-        diff = ctime > self.pctime
-        if self.probes and (diff or force):
-            bangline = re.compile('^#!(?P<interpreter>(/\w+)+)$')
-            for name, entry in [(name, entry) for name, entry in \
-                                self.probes.entries.iteritems() if entry.data]:
-                probe = lxml.etree.Element('probe')
-                probe.set('name', name )
-                probe.set('source', self.__name__)
-                probe.text = entry.data
-                match = bangline.match(entry.data.split('\n')[0])
-                if match:
-                    probe.set('interpreter', match.group('interpreter'))
-                else:
-                    probe.set('interpreter', '/bin/sh')
-                ret.append(probe)
-        return ret
+        if self.probes:
+            return self.probes.get_probe_data(meta)
+        return []
 
-    def ReceiveData(self, client, data):
+    def ReceiveData(self, client, datalist):
+        self.cgroups[client.hostname] = []
+        self.probedata[client.hostname] = {}
+        for data in datalist:
+            self.ReceiveDataItem(client, data)
+        self.pdirty = True
+        self.write_probedata()
+
+    def ReceiveDataItem(self, client, data):
         '''Receive probe results pertaining to client'''
         if not self.cgroups.has_key(client.hostname):
             self.cgroups[client.hostname] = []
         if data.text == None:
             self.logger.error("Got null response to probe %s from %s" % \
                               (data.get('name'), client.hostname))
+            try:
+                self.probedata[client.hostname].update({data.get('name'): ''})
+            except KeyError:
+                self.probedata[client.hostname] = {data.get('name'): ''}
             return
         dlines = data.text.split('\n')
         self.logger.debug("%s:probe:%s:%s" % (client.hostname, 
@@ -380,9 +524,6 @@ class Metadata(Bcfg2.Server.Plugin.Plugin):
             self.probedata[client.hostname].update({ data.get('name'):dtext })
         except KeyError:
             self.probedata[client.hostname] = { data.get('name'):dtext }
-        self.pdirty = True
-        self.ptimes[client.hostname] = time.time()
-        self.write_probedata()
 
     def AuthenticateConnection(self, user, password, address):
         '''This function checks user and password'''
@@ -438,3 +579,77 @@ class Metadata(Bcfg2.Server.Plugin.Plugin):
         if user != 'root':
             self.session_cache[address] = (time.time(), user)
         return True
+
+    def GetClientByGroup(self, group):
+        '''Return a list of clients that are in a given group'''
+        return [client for client in self.clients \
+                if group in self.groups[self.clients[client]][1]]
+
+    def GetClientByProfile(self, profile):
+        '''Return a list of clients that are members of a given profile'''
+        return [client for client in self.clients \
+                if self.clients[client] == profile]
+    
+    def viz(self, hosts, bundles, key, colors):
+        '''admin mode viz support'''
+        groups_tree = lxml.etree.parse(self.data + "/groups.xml")
+        groups = groups_tree.getroot()
+        categories = {'default':'grey83'}
+        instances = {}
+        viz_str = ""
+        egroups = groups.findall("Group") + groups.findall('.//Groups/Group')
+        for group in egroups:
+            if not categories.has_key(group.get('category')):
+                categories[group.get('category')] = colors.pop()
+            group.set('color', categories[group.get('category')])
+        if None in categories:
+            del categories[None]
+        if hosts:
+            clients = self.clients
+            for client, profile in clients.iteritems():
+                if instances.has_key(profile):
+                    instances[profile].append(client)
+                else:
+                    instances[profile] = [client]
+            for profile, clist in instances.iteritems():
+                clist.sort()
+                viz_str += '''\t"%s-instances" [ label="%s", shape="record" ];\n''' \
+                    % (profile, '|'.join(clist))
+                viz_str += '''\t"%s-instances" -> "group-%s";\n''' \
+                    % (profile, profile)
+        if bundles:
+            bundles = []
+            [bundles.append(bund.get('name')) \
+                 for bund in groups.findall('.//Bundle') \
+                 if bund.get('name') not in bundles]
+            bundles.sort()
+            for bundle in bundles:
+                viz_str +=  '''\t"bundle-%s" [ label="%s", shape="septagon"];\n''' \
+                    % (bundle, bundle)
+        gseen = []
+        for group in egroups:
+            if group.get('profile', 'false') == 'true':
+                style = "filled, bold"
+            else:
+                style = "filled"
+            gseen.append(group.get('name'))
+            viz_str += '\t"group-%s" [label="%s", style="%s", fillcolor=%s];\n' % \
+                (group.get('name'), group.get('name'), style, group.get('color'))
+            if bundles:
+                for bundle in group.findall('Bundle'):
+                    viz_str += '\t"group-%s" -> "bundle-%s";\n' % \
+                        (group.get('name'), bundle.get('name'))
+        gfmt = '\t"group-%s" [label="%s", style="filled", fillcolor="grey83"];\n'
+        for group in egroups:
+            for parent in group.findall('Group'):
+                if parent.get('name') not in gseen:
+                    viz_str += gfmt % (parent.get('name'), parent.get('name'))
+                    gseen.append(parent.get("name"))
+                viz_str += '\t"group-%s" -> "group-%s" ;\n' % \
+                    (group.get('name'), parent.get('name'))
+        if key:
+            for category in categories:
+                viz_str += '''\t"''' + category + '''" [label="''' + category + \
+                    '''", shape="record", style="filled", fillcolor=''' + \
+                    categories[category] + '''];\n'''
+        return viz_str
