@@ -1,10 +1,18 @@
 '''This module implements a package management scheme for all images'''
-__revision__ = '$Revision: 3655 $'
+__revision__ = '$Revision: 4953 $'
 
 import logging, re, Bcfg2.Server.Plugin
 
 logger = logging.getLogger('Bcfg2.Plugins.Pkgmgr')
 
+try:
+    pdlist = set
+except:
+    class pdlist(list):
+        def add(self, item):
+            if item not in self:
+                self.append(item)
+
 class FuzzyDict(dict):
     fuzzy = re.compile('(?P<name>.*):(?P<alist>\S+(,\S+)*)')
     def __getitem__(self, key):
@@ -53,7 +61,7 @@ class PNode(Bcfg2.Server.Plugin.INode):
     def __init__(self, data, pdict, parent=None):
         # copy local attributes to all child nodes if no local attribute exists
         if not pdict.has_key('Package'):
-            pdict['Package'] = []
+            pdict['Package'] = pdlist()
         for child in data.getchildren():
             for attr in [key for key in data.attrib.keys() \
                          if key != 'name' and not child.attrib.has_key(key)]:
@@ -67,7 +75,7 @@ class PNode(Bcfg2.Server.Plugin.INode):
             self.contents['Package'] = FuzzyDict()
         for pkg in data.findall('./Package'):
             if pkg.attrib.has_key('name') and pkg.get('name') not in pdict['Package']:
-                pdict['Package'].append(pkg.get('name'))
+                pdict['Package'].add(pkg.get('name'))
             if pkg.get('name') != None:
                 self.contents['Package'][pkg.get('name')] = {}
                 if pkg.getchildren():
@@ -102,7 +110,7 @@ class PNode(Bcfg2.Server.Plugin.INode):
                         if pkg.get('multiarch'):
                             self.contents['Package'][pkgname]['multiarch'] = pkg.get('multiarch')
                     if pkgname not in pdict['Package']:
-                        pdict['Package'].append(pkgname)
+                        pdict['Package'].add(pkgname)
                     if pkg.getchildren():
                         self.contents['Package'][pkgname]['__children__'] = pkg.getchildren()
                 else:
@@ -117,7 +125,7 @@ class PkgSrc(Bcfg2.Server.Plugin.XMLSrc):
 class Pkgmgr(Bcfg2.Server.Plugin.PrioDir):
     '''This is a generator that handles package assignments'''
     __name__ = 'Pkgmgr'
-    __version__ = '$Id: Pkgmgr.py 3655 2007-08-28 18:31:22Z desai $'
+    __version__ = '$Id: Pkgmgr.py 4953 2008-10-28 03:02:33Z desai $'
     __author__ = 'bcfg-dev@mcs.anl.gov'
     __child__ = PkgSrc
     __element__ = 'Package'
