@@ -1,16 +1,10 @@
 '''This module manages ssh key files for bcfg2'''
-__revision__ = '$Revision: 4210 $'
+__revision__ = '$Revision: 4880 $'
 
-import binascii, difflib, os, socket, xml.sax.saxutils
+import binascii, os, socket, tempfile
 import Bcfg2.Server.Plugin
 
-def update_file(path, diff):
-    '''Update file at path using diff'''
-    newdata = '\n'.join(difflib.restore(diff.split('\n'), 1))
-    print "writing file, %s" % path
-    open(path, 'w').write(newdata)
-
-class SSHbase(Bcfg2.Server.Plugin.Plugin,  Bcfg2.Server.Plugin.DirectoryBacked):
+class SSHbase(Bcfg2.Server.Plugin.GeneratorPlugin,  Bcfg2.Server.Plugin.DirectoryBacked):
     '''The sshbase generator manages ssh host keys (both v1 and v2)
     for hosts.  It also manages the ssh_known_hosts file. It can
     integrate host keys from other management domains and similarly
@@ -29,7 +23,7 @@ class SSHbase(Bcfg2.Server.Plugin.Plugin,  Bcfg2.Server.Plugin.DirectoryBacked):
       is regenerated each time a new key is generated.
 '''
     __name__ = 'SSHbase'
-    __version__ = '$Id: SSHbase.py 4210 2008-01-08 15:47:37Z bradshaw $'
+    __version__ = '$Id: SSHbase.py 4880 2008-08-19 03:11:18Z desai $'
     __author__ = 'bcfg-dev@mcs.anl.gov'
 
     pubkeys = ["ssh_host_dsa_key.pub.H_%s",
@@ -56,32 +50,61 @@ class SSHbase(Bcfg2.Server.Plugin.Plugin,  Bcfg2.Server.Plugin.DirectoryBacked):
                          '/etc/ssh/ssh_host_key':self.build_hk,
                          '/etc/ssh/ssh_host_key.pub':self.build_hk}}
         self.ipcache = {}
-        self.__rmi__ = ['GetPubKeys']
+        self.__skn = False
+
+    def get_skn(self):
+        '''build memory cache of the ssh known hosts file'''
+        if not self.__skn:
+            static_entries = [key for key in self.entries \
+                              if key.endswith('.static')]
+            if static_entries:
+                self.__skn = "\n".join([self.entries[key].data for \
+                                        key in static_entries])
+            else:
+                self.__skn = ''
+            pubkeys = [pubk for pubk in self.entries.keys() \
+                       if pubk.find('.pub.H_') != -1]
+            pubkeys.sort()
+            for pubkey in pubkeys:
+                hostname = pubkey.split('H_')[1]
+                try:
+                    (ipaddr, fqdn) = self.get_ipcache_entry(hostname)
+                except socket.gaierror:
+                    continue
+                shortname = hostname.split('.')[0]
+                self.__skn += "%s,%s,%s %s" % (shortname, fqdn, ipaddr,
+                                               self.entries[pubkey].data)
+        return self.__skn
+
+    def set_skn(self, value):
+        '''Set backing data for skn'''
+        self.__skn = value
+    skn = property(get_skn, set_skn)
 
     def HandleEvent(self, event=None):
         '''Local event handler that does skn regen on pubkey change'''
         Bcfg2.Server.Plugin.DirectoryBacked.HandleEvent(self, event)
-        if (len(self.entries.keys())) > (0.90 * len(os.listdir(self.data))) and \
-               event and '_key.pub.H_' in event.filename:
-            self.cache_skn()
-        elif (len(self.entries.keys())) > (0.90 * len(os.listdir(self.data))) and \
-             not hasattr(self, 'static_skn'):
-            self.cache_skn()
+        if event and '_key.pub.H_' in event.filename:
+            self.skn = False
+        if event and event.filename.endswith('.static'):
+            self.skn = False
+        if not self.skn:
+            if (len(self.entries.keys())) > (0.95 * len(os.listdir(self.data))):
+                a = self.skn
 
     def HandlesEntry(self, entry):
         '''Handle key entries dynamically'''
         return entry.tag == 'ConfigFile' and \
-               [fpat for fpat in self.keypatterns if entry.get('name').endswith(fpat)]
+               ([fpat for fpat in self.keypatterns
+                 if entry.get('name').endswith(fpat)]
+                or entry.get('name').endswith('ssh_known_hosts'))
 
     def HandleEntry(self, entry, metadata):
-        '''Bind key data'''
-        return self.build_hk(entry, metadata)
-
-    def GetPubKeys(self, _):
-        '''Export public key data'''
-        if not hasattr(self, 'static_skn'):
-            self.cache_skn()
-        return self.static_skn
+        '''Bind data'''
+        if entry.get('name').endswith('ssh_known_hosts'):
+            return self.build_skn(entry, metadata)
+        else:
+            return self.build_hk(entry, metadata)
 
     def get_ipcache_entry(self, client):
         '''build a cache of dns results'''
@@ -105,27 +128,10 @@ class SSHbase(Bcfg2.Server.Plugin.Plugin,  Bcfg2.Server.Plugin.DirectoryBacked):
                 self.logger.error("Failed to find IP address for %s" % client)
                 raise socket.gaierror
 
-    def cache_skn(self):
-        '''build memory cache of the ssh known hosts file'''
-        self.static_skn = ''
-        pubkeys = [pubk for pubk in self.entries.keys() if pubk.find('.pub.H_') != -1]
-        pubkeys.sort()
-        for pubkey in pubkeys:
-            hostname = pubkey.split('H_')[1]
-            try:
-                (ipaddr, fqdn) = self.get_ipcache_entry(hostname)
-            except socket.gaierror:
-                continue
-            shortname = hostname.split('.')[0]
-            self.static_skn += "%s,%s,%s %s" % (shortname, fqdn, ipaddr,
-                                         self.entries[pubkey].data)
-
     def build_skn(self, entry, metadata):
         '''This function builds builds a host specific known_hosts file'''
         client = metadata.hostname
-        if not hasattr(self, 'static_skn'):
-            self.cache_skn()
-        entry.text = self.static_skn
+        entry.text = self.skn
         hostkeys = [keytmpl % client for keytmpl in self.pubkeys \
                         if self.entries.has_key(keytmpl % client)]
         hostkeys.sort()
@@ -170,7 +176,8 @@ class SSHbase(Bcfg2.Server.Plugin.Plugin,  Bcfg2.Server.Plugin.DirectoryBacked):
             if hostkey not in self.entries.keys():
                 fileloc = "%s/%s" % (self.data, hostkey)
                 publoc = self.data + '/' + ".".join([hostkey.split('.')[0]]+['pub', "H_%s" % client])
-                temploc =  "/tmp/%s" % hostkey
+                tempdir = tempfile.mkdtemp()
+                temploc = "%s/%s" % (tempdir, hostkey)
                 os.system('ssh-keygen -q -f %s -N "" -t %s -C root@%s < /dev/null' %
                           (temploc, keytype, client))
                 open(fileloc, 'w').write(open(temploc).read())
@@ -180,20 +187,18 @@ class SSHbase(Bcfg2.Server.Plugin.Plugin,  Bcfg2.Server.Plugin.DirectoryBacked):
                 try:
                     os.unlink(temploc)
                     os.unlink("%s.pub" % temploc)
+                    os.rmdir(tempdir)
                 except OSError:
                     self.logger.error("Failed to unlink temporary ssh keys")
 
-    def AcceptEntry(self, meta, _, entry_name, diff, fulldata, metadata_updates={}):
-        '''per-plugin bcfg2-admin pull support'''
-        filename = "%s/%s.H_%s" % (self.data, entry_name.split('/')[-1],
-                                   meta.hostname)
-        print "This file will be installed as file %s" % filename
-        if raw_input("Should it be installed? (N/y): ") in ['Y', 'y']:
-            print "writing file, %s" % filename
-            if fulldata:
-                newdata = fulldata
-            else:
-                newdata = '\n'.join(difflib.restore(diff.split('\n'), 1))
-            open(filename, 'w').write(newdata)
+    def AcceptChoices(self, _, metadata):
+        return [Bcfg2.Server.Plugin.Specificity(hostname=metadata.hostname)]
 
-        
+    def AcceptPullData(self, specific, entry, log):
+        '''per-plugin bcfg2-admin pull support'''
+        # specific will always be host specific
+        filename = "%s/%s.H_%s" % (self.data, entry['name'].split('/')[-1],
+                                   specific.hostname)
+        open(filename, 'w').write(entry['text'])
+        if log:
+            print "Wrote file %s" % filename
